﻿#include "stdafx.h"
#include "APIExample.h"
#include "CAgoraOriginalAudioDlg.h"



IMPLEMENT_DYNAMIC(CAgoraOriginalAudioDlg, CDialogEx)

CAgoraOriginalAudioDlg::CAgoraOriginalAudioDlg(CWnd* pParent /*=nullptr*/)
	: CDialogEx(IDD_DIALOG_ORIGINAL_AUDIO_, pParent)
{

}

CAgoraOriginalAudioDlg::~CAgoraOriginalAudioDlg()
{
}

void CAgoraOriginalAudioDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_STATIC_VIDEO, m_staVideoArea);
	DDX_Control(pDX, IDC_LIST_INFO_BROADCASTING, m_lstInfo);
	DDX_Control(pDX, IDC_STATIC_CHANNELNAME, m_staChannel);
	DDX_Control(pDX, IDC_EDIT_CHANNELNAME, m_edtChannel);
	DDX_Control(pDX, IDC_BUTTON_JOINCHANNEL, m_btnJoinChannel);
	DDX_Control(pDX, IDC_STATIC_ORIGINAL_AUDIO, m_staOriginalAudio);
	DDX_Control(pDX, IDC_COMBO_ORIGINAL_AUDIO, m_cmbOriginalAudio);
	DDX_Control(pDX, IDC_BUTTON_SET_AUDIO_PROC, m_btnSetAudioProc);
	DDX_Control(pDX, IDC_STATIC_DETAIL, m_staDetail);
	DDX_Control(pDX, IDC_BUTTON_ENCODED_AUDIO, m_btnEncodedAudio);
}


BEGIN_MESSAGE_MAP(CAgoraOriginalAudioDlg, CDialogEx)
	ON_WM_SHOWWINDOW()
	ON_MESSAGE(WM_MSGID(EID_JOINCHANNEL_SUCCESS), &CAgoraOriginalAudioDlg::OnEIDJoinChannelSuccess)
	ON_MESSAGE(WM_MSGID(EID_LEAVE_CHANNEL), &CAgoraOriginalAudioDlg::OnEIDLeaveChannel)
	ON_MESSAGE(WM_MSGID(EID_USER_JOINED), &CAgoraOriginalAudioDlg::OnEIDUserJoined)
	ON_MESSAGE(WM_MSGID(EID_USER_OFFLINE), &CAgoraOriginalAudioDlg::OnEIDUserOffline)
	ON_MESSAGE(WM_MSGID(EID_REMOTE_VIDEO_STATE_CHANGED), &CAgoraOriginalAudioDlg::OnEIDRemoteVideoStateChanged)
	ON_BN_CLICKED(IDC_BUTTON_JOINCHANNEL, &CAgoraOriginalAudioDlg::OnBnClickedButtonJoinchannel)
	ON_BN_CLICKED(IDC_BUTTON_SET_AUDIO_PROC, &CAgoraOriginalAudioDlg::OnBnClickedButtonSetOriginalProc)
	ON_LBN_SELCHANGE(IDC_LIST_INFO_BROADCASTING, &CAgoraOriginalAudioDlg::OnSelchangeListInfoBroadcasting)
	ON_BN_CLICKED(IDC_BUTTON_ENCODED_AUDIO, &CAgoraOriginalAudioDlg::OnBnClickedButtonEncodedAudio)
END_MESSAGE_MAP()


bool COriginalAudioProcFrameObserver::onRecordAudioFrame(const char* channelId, AudioFrame& audioFrame)
{
	SIZE_T nSize = audioFrame.channels * audioFrame.samplesPerChannel * 2;
	unsigned int readByte = 0;
	int timestamp = GetTickCount();
	short *pBuffer = (short *)audioFrame.buffer;
	for (SIZE_T i = 0; i < nSize / 2; i++)
	{
		if (pBuffer[i] * 2 > 32767) {
			pBuffer[i] = 32767;
		}
		else if (pBuffer[i] * 2 < -32768) {
			pBuffer[i] = -32768;
		}
		else {
			pBuffer[i] *= 2;
		}
	}
#ifdef _DEBUG
	CString strInfo;
	strInfo.Format(_T("audio Frame buffer size:%d, timestamp:%d \n"), nSize, timestamp);
	OutputDebugString(strInfo);
	audioFrame.renderTimeMs = timestamp;
#endif
	return true;
}
/*
	Get the sound played.
	parameter:
	audioFrame:Audio naked data.
	See: AudioFrame
	return
	True: Buffer data in AudioFrame is valid, the data will be sent;
	False: The buffer data in the AudioFrame is invalid and will be discarded.
*/
bool COriginalAudioProcFrameObserver::onPlaybackAudioFrame(const char* channelId, AudioFrame& audioFrame)
{
	return true;
}
/*
	Gets the data after recording and playing the voice mix.
	annotations:
		This method returns only single-channel data.
	parameter:
	audioFrame Audio naked data. See: AudioFrame
	return:
	True: Buffer data in AudioFrame is valid, the data will be sent;
	False: The buffer data in the AudioFrame is invalid and will be discarded.
*/
bool COriginalAudioProcFrameObserver::onMixedAudioFrame(const char* channelId, AudioFrame& audioFrame)
{
	return true;
}
/*
	Gets the specified user's voice before the mix.
	parameter:
	uid: Specifies the user ID of the user.
	audioFrame: Audio naked data. See: AudioFrame.
	return:
	True: Buffer data in AudioFrame is valid, the data will be sent;
	False: The buffer data in the AudioFrame is invalid and will be discarded.
*/
bool COriginalAudioProcFrameObserver::onPlaybackAudioFrameBeforeMixing(const char* channelId, media::base::user_id_t uid, AudioFrame& audioFrame)
{
	return true;
}

bool COriginalAudioProcFrameObserver::onPlaybackAudioFrameBeforeMixing(const char* channelId, rtc::uid_t uid, AudioFrame& audioFrame)
{
	return true;
}

int COriginalAudioProcFrameObserver::getObservedAudioFramePosition()
{
	//agora::media::IAudioFrameObserverBase::AUDIO_FRAME_POSITION pos = agora::media::IAudioFrameObserverBase::AUDIO_FRAME_POSITION_NONE;
	return agora::media::IAudioFrameObserverBase::AUDIO_FRAME_POSITION_MIXED |
		agora::media::IAudioFrameObserverBase::AUDIO_FRAME_POSITION_PLAYBACK |
		agora::media::IAudioFrameObserverBase::AUDIO_FRAME_POSITION_RECORD;
}

agora::media::IAudioFrameObserverBase::AudioParams COriginalAudioProcFrameObserver::getPlaybackAudioParams() {
	agora::media::IAudioFrameObserverBase::AudioParams params(16000, 2, agora::rtc::RAW_AUDIO_FRAME_OP_MODE_READ_ONLY, 320);
	return params;
}

agora::media::IAudioFrameObserverBase::AudioParams COriginalAudioProcFrameObserver::getRecordAudioParams() {
	agora::media::IAudioFrameObserverBase::AudioParams params(16000, 2, agora::rtc::RAW_AUDIO_FRAME_OP_MODE_READ_ONLY, 320);
	return params;
}

agora::media::IAudioFrameObserverBase::AudioParams COriginalAudioProcFrameObserver::getMixedAudioParams() {
	agora::media::IAudioFrameObserverBase::AudioParams params(16000, 2, agora::rtc::RAW_AUDIO_FRAME_OP_MODE_READ_ONLY, 320);
	return params;
}


bool COriginalAudioProcFrameObserver::onEarMonitoringAudioFrame(AudioFrame& audioFrame)
{
	return FALSE;
}

agora::media::IAudioFrameObserverBase::AudioParams COriginalAudioProcFrameObserver::getEarMonitoringAudioParams()
{
	return AudioParams();
}

//Initialize the Ctrl Text.
void CAgoraOriginalAudioDlg::InitCtrlText()
{
	m_staChannel.SetWindowText(commonCtrlChannel);
	m_btnJoinChannel.SetWindowText(commonCtrlJoinChannel);
	m_staOriginalAudio.SetWindowText(OriginalAudioCtrlProc);
	m_btnSetAudioProc.SetWindowText(OriginalAudioCtrlSetProc);
}


//Initialize the Agora SDK
bool CAgoraOriginalAudioDlg::InitAgora()
{
	//create Agora RTC engine
	m_rtcEngine = createAgoraRtcEngine();
	if (!m_rtcEngine) {
		m_lstInfo.InsertString(m_lstInfo.GetCount() - 1, _T("createAgoraRtcEngine failed"));
		return false;
	}
	//set message notify receiver window
	m_eventHandler.SetMsgReceiver(m_hWnd);

	RtcEngineContext context;
	std::string strAppID = GET_APP_ID;
	context.appId = strAppID.c_str();
	context.eventHandler = &m_eventHandler;
	//set channel profile in the engine to the CHANNEL_PROFILE_LIVE_BROADCASTING.
	context.channelProfile = CHANNEL_PROFILE_LIVE_BROADCASTING;
	//initialize the Agora RTC engine context.
	int ret = m_rtcEngine->initialize(context);
	if (ret != 0) {
		m_initialize = false;
		CString strInfo;
		strInfo.Format(_T("initialize failed: %d"), ret);
		m_lstInfo.InsertString(m_lstInfo.GetCount(), strInfo);
		return false;
	}
	else
		m_initialize = true;
	m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("initialize success"));
	//enable video in the engine.
	m_rtcEngine->enableAudio();
	m_rtcEngine->enableVideo();
	m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("enable video"));
	
	m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("live broadcasting"));
	//set client role in the engine to the CLIENT_ROLE_BROADCASTER.
	m_rtcEngine->setClientRole(CLIENT_ROLE_BROADCASTER);
	m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("setClientRole broadcaster"));
	return true;
}


//UnInitialize the Agora SDK
void CAgoraOriginalAudioDlg::UnInitAgora()
{
	if (m_rtcEngine) {
		if (m_joinChannel)
			//leave channel
			m_joinChannel = !m_rtcEngine->leaveChannel();
		//stop preview in the engine.
		m_rtcEngine->stopPreview();
		m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("stopPreview"));
		//disable video in the engine.
		m_rtcEngine->disableVideo();
		m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("disableVideo"));
		RegisterAudioFrameObserver(FALSE);
		if (bRegisterEncoded) {
			AudioEncodedFrameObserverConfig config;
			m_rtcEngine->registerAudioEncodedFrameObserver(config, nullptr);
			m_btnEncodedAudio.SetWindowText(L"encoded Audio");
			bRegisterEncoded = false;
		}
		//release engine.
		m_rtcEngine->release(true);
		m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("release rtc engine"));
		m_rtcEngine = NULL;
	}
}

//render local video from SDK local capture.
void CAgoraOriginalAudioDlg::RenderLocalVideo()
{
	if (m_rtcEngine) {

		VideoCanvas canvas;
		canvas.renderMode = media::base::RENDER_MODE_FIT;
		canvas.uid = 0;
		canvas.view = m_localVideoWnd.GetSafeHwnd();
		//setup local video in the engine to canvas.
		m_rtcEngine->setupLocalVideo(canvas);
		m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("setupLocalVideo"));
		//start preview in the engine.
		m_rtcEngine->startPreview();
		m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("startPreview"));
	}
}


//resume window status
void CAgoraOriginalAudioDlg::ResumeStatus()
{
	InitCtrlText();
	m_staDetail.SetWindowText(_T(""));
	m_edtChannel.SetWindowText(_T(""));
	m_cmbOriginalAudio.SetCurSel(0);
	m_btnJoinChannel.EnableWindow(FALSE);
	m_lstInfo.ResetContent();
	m_joinChannel = false;
	m_initialize = false;
	m_setAudioProc = false;
}

void CAgoraOriginalAudioDlg::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialogEx::OnShowWindow(bShow, nStatus);
	if (bShow)//bShwo is true ,show window 
	{
		InitCtrlText();
		RenderLocalVideo();
	}
	else {
		ResumeStatus();
	}
}


BOOL CAgoraOriginalAudioDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();
	m_localVideoWnd.Create(NULL, NULL, WS_CHILD | WS_VISIBLE | WS_BORDER | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0, 0, 1, 1), this, ID_BASEWND_VIDEO + 100);
	RECT rcArea;
	m_staVideoArea.GetClientRect(&rcArea);
	m_localVideoWnd.MoveWindow(&rcArea);
	m_localVideoWnd.ShowWindow(SW_SHOW);

	int i = 0;
	m_mapAudioFrame.insert(std::make_pair(_T("amplification"), &m_originalAudioProcFrameObserver));
	m_cmbOriginalAudio.InsertString(i++, _T("amplification"));
	ResumeStatus();
	return TRUE;
}


BOOL CAgoraOriginalAudioDlg::PreTranslateMessage(MSG* pMsg)
{
	if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_RETURN) {
		return TRUE;
	}
	return CDialogEx::PreTranslateMessage(pMsg);
}


void CAgoraOriginalAudioDlg::OnBnClickedButtonJoinchannel()
{
	if (!m_rtcEngine || !m_initialize)
		return;
	CString strInfo;
	if (!m_joinChannel) {
		CString strChannelName;
		m_edtChannel.GetWindowText(strChannelName);
		if (strChannelName.IsEmpty()) {
			AfxMessageBox(_T("Fill channel name first"));
			return;
		}
		std::string szChannelId = cs2utf8(strChannelName);
		ChannelMediaOptions options;
		options.channelProfile = CHANNEL_PROFILE_LIVE_BROADCASTING;
		options.clientRoleType = CLIENT_ROLE_BROADCASTER;
		//join channel in the engine.
		if (0 == m_rtcEngine->joinChannel(APP_TOKEN, szChannelId.c_str(), 0, options)) {
			strInfo.Format(_T("join channel %s, use ChannelMediaOptions"), getCurrentTime());
			m_btnJoinChannel.EnableWindow(FALSE);
		}
	}
	else {
		//leave channel in the engine.
		if (0 == m_rtcEngine->leaveChannel()) {
			strInfo.Format(_T("leave channel %s"), getCurrentTime());
		}
	}
	m_lstInfo.InsertString(m_lstInfo.GetCount(), strInfo);
}

/*
	register or unregister agora audio Frame Observer.
*/
BOOL CAgoraOriginalAudioDlg::RegisterAudioFrameObserver(BOOL bEnable, IAudioFrameObserver *audioFrameObserver)
{
	agora::util::AutoPtr<agora::media::IMediaEngine> mediaEngine;
	//query interface agora::AGORA_IID_MEDIA_ENGINE in the engine.
	mediaEngine.queryInterface(m_rtcEngine, agora::rtc::AGORA_IID_MEDIA_ENGINE);
	int nRet = 0;
	if (mediaEngine.get() == NULL)
		return FALSE;
	//register audio frame observer.
	if (bEnable)
		nRet = mediaEngine->registerAudioFrameObserver(audioFrameObserver);
	else
		//unregister audio frame observer.
		nRet = mediaEngine->registerAudioFrameObserver(NULL);

	return nRet == 0 ? TRUE : FALSE;
}

//setOriginalProc button handler.
void CAgoraOriginalAudioDlg::OnBnClickedButtonSetOriginalProc()
{
	if (!m_setAudioProc)
	{
		CString strInfo;
		CString strAudioProc;
		m_cmbOriginalAudio.GetWindowText(strAudioProc);
		//register audio frame observer.
		RegisterAudioFrameObserver(TRUE, m_mapAudioFrame[strAudioProc]);
		m_rtcEngine->setRecordingAudioFrameParameters(44100, 2, RAW_AUDIO_FRAME_OP_MODE_READ_WRITE, 1024);
		//m_btnSetAudioProc.SetWindowText(OriginalAudioCtrlUnSetProc);
		strInfo.Format(_T("register %s auido frame obsever"), strAudioProc);
		m_lstInfo.InsertString(m_lstInfo.GetCount(), strInfo);
		m_btnJoinChannel.EnableWindow(TRUE);
	}
	/*else {
		//unregister audio frame observer.
		RegisterAudioFrameObserver(FALSE, NULL);
		m_btnSetAudioProc.SetWindowText(OriginalAudioCtrlSetProc);
		m_lstInfo.InsertString(m_lstInfo.GetCount(), _T("unregister audio frame observer"));
	}
	m_setAudioProc = !m_setAudioProc;*/
}

//select list item handler
void CAgoraOriginalAudioDlg::OnSelchangeListInfoBroadcasting()
{
	int sel = m_lstInfo.GetCurSel();
	if (sel < 0)return;
	CString strDetail;
	m_lstInfo.GetText(sel, strDetail);
	m_staDetail.SetWindowText(strDetail);
}



//EID_JOINCHANNEL_SUCCESS message window handler
LRESULT CAgoraOriginalAudioDlg::OnEIDJoinChannelSuccess(WPARAM wParam, LPARAM lParam)
{
	m_joinChannel = true;
	m_btnJoinChannel.SetWindowText(commonCtrlLeaveChannel);
	m_btnJoinChannel.EnableWindow(TRUE);
	CString strInfo;
	strInfo.Format(_T("%s:join success, uid=%u"), getCurrentTime(), wParam);
	m_lstInfo.InsertString(m_lstInfo.GetCount(), strInfo);
	m_localVideoWnd.SetUID(wParam);
	//notify parent window
	::PostMessage(GetParent()->GetSafeHwnd(), WM_MSGID(EID_JOINCHANNEL_SUCCESS), TRUE, 0);
	return 0;
}

//EID_LEAVEHANNEL_SUCCESS message window handler
LRESULT CAgoraOriginalAudioDlg::OnEIDLeaveChannel(WPARAM wParam, LPARAM lParam)
{
	m_joinChannel = false;
	m_btnJoinChannel.SetWindowText(commonCtrlJoinChannel);
	CString strInfo;
	strInfo.Format(_T("leave channel success %s"), getCurrentTime());
	m_lstInfo.InsertString(m_lstInfo.GetCount(), strInfo);
	::PostMessage(GetParent()->GetSafeHwnd(), WM_MSGID(EID_JOINCHANNEL_SUCCESS), FALSE, 0);
	return 0;
}

//EID_USER_JOINED message window handler
LRESULT CAgoraOriginalAudioDlg::OnEIDUserJoined(WPARAM wParam, LPARAM lParam)
{
	CString strInfo;
	strInfo.Format(_T("%u joined"), wParam);
	m_lstInfo.InsertString(m_lstInfo.GetCount(), strInfo);
	return 0;
}

//EID_USER_OFFLINE message handler.
LRESULT CAgoraOriginalAudioDlg::OnEIDUserOffline(WPARAM wParam, LPARAM lParam)
{
	uid_t remoteUid = (uid_t)wParam;
	CString strInfo;
	strInfo.Format(_T("%u offline, reason:%d"), remoteUid, lParam);
	m_lstInfo.InsertString(m_lstInfo.GetCount(), strInfo);
	return 0;
}

//EID_REMOTE_VIDEO_STATE_CHANGED message window handler.
LRESULT CAgoraOriginalAudioDlg::OnEIDRemoteVideoStateChanged(WPARAM wParam, LPARAM lParam)
{
	PVideoStateStateChanged stateChanged = (PVideoStateStateChanged)wParam;
	if (stateChanged) {
		//onRemoteVideoStateChanged
		CString strSateInfo;
		switch (stateChanged->state) {
		case REMOTE_VIDEO_STATE_STARTING:
			strSateInfo = _T("REMOTE_VIDEO_STATE_STARTING");
			break;
		case REMOTE_VIDEO_STATE_STOPPED:
			strSateInfo = _T("strSateInfo");
			break;
		case REMOTE_VIDEO_STATE_DECODING:
			strSateInfo = _T("REMOTE_VIDEO_STATE_DECODING");
			break;
		case REMOTE_VIDEO_STATE_FAILED:
			strSateInfo = _T("REMOTE_VIDEO_STATE_FAILED ");
			break;
		case REMOTE_VIDEO_STATE_FROZEN:
			strSateInfo = _T("REMOTE_VIDEO_STATE_FROZEN  ");
			break;
		}
		CString strInfo;
		strInfo.Format(_T("onRemoteVideoStateChanged: uid=%u, %s"), stateChanged->uid, strSateInfo);
		m_lstInfo.InsertString(m_lstInfo.GetCount(), strInfo);
	}
	return 0;
}



/*
note:
	Join the channel callback.This callback method indicates that the client
	successfully joined the specified channel.Channel ids are assigned based
	on the channel name specified in the joinChannel. If IRtcEngine::joinChannel
	is called without a user ID specified. The server will automatically assign one
parameters:
	channel:channel name.
	uid: user ID。If the UID is specified in the joinChannel, that ID is returned here;
	Otherwise, use the ID automatically assigned by the Agora server.
	elapsed: The Time from the joinChannel until this event occurred (ms).
*/
void COriginalAudioEventHandler::onJoinChannelSuccess(const char* channel, uid_t uid, int elapsed)
{
	if (m_hMsgHanlder) {
		::PostMessage(m_hMsgHanlder, WM_MSGID(EID_JOINCHANNEL_SUCCESS), (WPARAM)uid, (LPARAM)elapsed);
	}
}
/*
note:
	In the live broadcast scene, each anchor can receive the callback
	of the new anchor joining the channel, and can obtain the uID of the anchor.
	Viewers also receive a callback when a new anchor joins the channel and
	get the anchor's UID.When the Web side joins the live channel, the SDK will
	default to the Web side as long as there is a push stream on the
	Web side and trigger the callback.
parameters:
	uid: remote user/anchor ID for newly added channel.
	elapsed: The joinChannel is called from the local user to the delay triggered
	by the callback（ms).
*/
void COriginalAudioEventHandler::onUserJoined(uid_t uid, int elapsed)
{
	if (m_hMsgHanlder) {
		::PostMessage(m_hMsgHanlder, WM_MSGID(EID_USER_JOINED), (WPARAM)uid, (LPARAM)elapsed);
	}
}

/*
note:
	Remote user (communication scenario)/anchor (live scenario) is called back from
	the current channel.A remote user/anchor has left the channel (or dropped the line).
	There are two reasons for users to leave the channel, namely normal departure and
	time-out:When leaving normally, the remote user/anchor will send a message like
	"goodbye". After receiving this message, determine if the user left the channel.
	The basis of timeout dropout is that within a certain period of time
	(live broadcast scene has a slight delay), if the user does not receive any
	packet from the other side, it will be judged as the other side dropout.
	False positives are possible when the network is poor. We recommend using the
	Agora Real-time messaging SDK for reliable drop detection.
parameters:
	uid: The user ID of an offline user or anchor.
	reason:Offline reason: USER_OFFLINE_REASON_TYPE.
*/
void COriginalAudioEventHandler::onUserOffline(uid_t uid, USER_OFFLINE_REASON_TYPE reason)
{
	if (m_hMsgHanlder) {
		::PostMessage(m_hMsgHanlder, WM_MSGID(EID_USER_OFFLINE), (WPARAM)uid, (LPARAM)reason);
	}
}
/*
note:
	When the App calls the leaveChannel method, the SDK indicates that the App
	has successfully left the channel. In this callback method, the App can get
	the total call time, the data traffic sent and received by THE SDK and other
	information. The App obtains the call duration and data statistics received
	or sent by the SDK through this callback.
parameters:
	stats: Call statistics.
*/

void COriginalAudioEventHandler::onLeaveChannel(const RtcStats& stats)
{
	if (m_hMsgHanlder) {
		::PostMessage(m_hMsgHanlder, WM_MSGID(EID_LEAVE_CHANNEL), 0, 0);
	}
}
/**
	Occurs when the remote video state changes.
	@note This callback does not work properly when the number of users (in the Communication profile) or broadcasters (in the Live-broadcast profile) in the channel exceeds 17.

	@param uid ID of the remote user whose video state changes.
	@param state State of the remote video. See #REMOTE_VIDEO_STATE.
	@param reason The reason of the remote video state change. See
	#REMOTE_VIDEO_STATE_REASON.
	@param elapsed Time elapsed (ms) from the local user calling the
	\ref agora::rtc::IRtcEngine::joinChannel "joinChannel" method until the
	SDK triggers this callback.
*/
void COriginalAudioEventHandler::onRemoteVideoStateChanged(uid_t uid, REMOTE_VIDEO_STATE state, REMOTE_VIDEO_STATE_REASON reason, int elapsed)
{
	if (m_hMsgHanlder) {
		PVideoStateStateChanged stateChanged = new VideoStateStateChanged;
		stateChanged->uid = uid;
		stateChanged->reason = reason;
		stateChanged->state = state;
		::PostMessage(m_hMsgHanlder, WM_MSGID(EID_REMOTE_VIDEO_STATE_CHANGED), (WPARAM)stateChanged, 0);
	}
}


void CAudioEncodedFrameObserver::onRecordAudioEncodedFrame(const uint8_t* frameBuffer, int length, const EncodedAudioFrameInfo& audioEncodedFrameInfo)
{
	//to do...
}

void CAudioEncodedFrameObserver::onPlaybackAudioEncodedFrame(const uint8_t* frameBuffer, int length, const EncodedAudioFrameInfo& audioEncodedFrameInfo)
{
	//to do...
}

void CAudioEncodedFrameObserver::onMixedAudioEncodedFrame(const uint8_t* frameBuffer, int length, const EncodedAudioFrameInfo& audioEncodedFrameInfo)
{
	//to do...
}

void CAgoraOriginalAudioDlg::OnBnClickedButtonEncodedAudio()
{
	AudioEncodedFrameObserverConfig config;
	if (!bRegisterEncoded) {
		m_rtcEngine->registerAudioEncodedFrameObserver(config, &encodeObserver);
		m_btnEncodedAudio.SetWindowText(L"cancel encoded Audio");
	}
	else {
		m_rtcEngine->registerAudioEncodedFrameObserver(config, nullptr);
		m_btnEncodedAudio.SetWindowText(L"encoded Audio");
	}
	bRegisterEncoded = !bRegisterEncoded;
}
